/*
 * Copyright (c) 1993, 1994, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that (i) the above copyright notices and this
 * permission notice appear in all copies of the software and related
 * documentation, and (ii) the name of Silicon Graphics may not be
 * used in any advertising or publicity relating to the software
 * without the specific, prior written permission of Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#include "uLinMath.h" /* uSmoke.h is nested inside */
#include "smoke_ext.h" /* extern'd variables used in this module */


Matrix ident4 = {1.0, 0.0, 0.0, 0.0,
                 0.0, 1.0, 0.0, 0.0,
                 0.0, 0.0, 1.0, 0.0,
                 0.0, 0.0, 0.0, 1.0};

float material[] = { DIFFUSE, 1, 1, 0, AMBIENT, 0, 0, 0, ALPHA, .75, LMNULL };

float two[] = { AMBIENT,0.2, 0.2, 0.2,
		ATTENUATION,1.0, 0.0,
   	        TWOSIDE , 0.0, LMNULL};

float lite[] = {POSITION, 0,0,-1,0,LMNULL};

def_simple_light_calc(){
        lmdef(DEFMATERIAL, 1, 0, material);
        lmdef(DEFLIGHT, 1, 0, lite);
        lmdef(DEFLMODEL, 1, 0, two);
}

use_simple_light_calc(){
        lmbind(MATERIAL,1);
        lmbind(LIGHT0, 1);
        lmbind(LMODEL, 1);
}

/* 
 * Parameters for texturing.
 */
float texps[] = 
   {  TX_MAGFILTER, TX_BILINEAR, 
      TX_MINFILTER, TX_MIPMAP_BILINEAR, 
      TX_WRAP_S, TX_CLAMP, 
      TX_WRAP_T, TX_CLAMP, 
      TX_NULL
   };

float tevps[] = {TV_MODULATE,TV_NULL};


static float vees[]={-1.0,-1.0, 0.0,
                      1.0,-1.0, 0.0,
                      1.0, 1.0, 0.0,
                     -1.0, 1.0, 0.0,
                      0.0, 1.0, 0.0 };

static float tees[]={0.0,0.0,0.0,1.0,1.0,1.0,1.0,0.0};

long arr[128][128];
long *image;

void add_fire_tex(void)
{
    fireTex = uNewTex();
    uLoadTexFile(fireTex, "fire.texture");

    texdef2d(fireTex->index,fireTex->comp,fireTex->sx,fireTex->sy,
	     (unsigned long *)fireTex->image,0,texps);
}

void inittex(void)
{

    smokeTex = uNewTex();
    uLoadTexFile(smokeTex, "smoke.texture");

    texdef2d(smokeTex->index,smokeTex->comp,smokeTex->sx,smokeTex->sy,
	     (unsigned long *)smokeTex->image,0,texps);

    add_fire_tex();

    tevdef(1,2,tevps);
    /*texbind(0,smokeTex->index); */ /* testing uApplyTex?*/
    tevbind(0,1);
}


int     xangle = 0;              /* counter for angle rotation */
int     yangle = 0;              /* counter for angle rotation */
int     zangle = 0;              /* counter for angle rotation */
int        X=0,Y=0,Z=0;

#include <gl/device.h>
static uVec3 eye;
extern float warp_factor;
unsigned long bkgd = 0x00000000; /*black by default*/

void usage(char *name)
{
   fprintf(stdout, "Usage: %s accepts the following comand line options:\n",name);
   fprintf(stdout, "              -bgn r g b, -end r g b {set bgn/end color of smoke plumes}\n");
   fprintf(stdout, "              -whi[te]               {set background to white}\n");
   fprintf(stdout, "              -c N                   {N number of smoke plumes (3 by default)\n");
   fprintf(stdout, "              -exp F 		 {puff expansion F-actor}\n");
   fprintf(stdout, "              -dir x y z 		 {smoke direction vector}\n");

}

void main(int argc, char **argv)
{
    int stype=-1,bset=0,eset=0,i,k=3;
    short attached = 1;
    short value;
    int dev,j;
    
    uVec3 red, blu,dirvec, forg;
    uSmoke *tmp_smoke;
    uSmoke *tmp_fire;
    float sspeed =    1.73; /*type smoke numbers*/
    float dissipate = 2.42; /*type smoke numbers*/
    float density   = .33;
    float expand   =  1.83;
    int   set_dense = 0;

    uSetVec3(red,0.01,0.01,0.01); /*default bgn colr is black*/
    uSetVec3(blu,1.0,1.0,1.0);    /*default end colr is white*/
    uSetVec3(eye,0,0,-1);         /* default eye vector */
    uSetVec3(dirvec,0,.9,0);       /* default direction vector */

   if(argc < 2) usage(argv[0]);
   for(j=1;j<argc;j++)
   {
   /* TYPE of SMOKE */
     if(!strncasecmp("-t",argv[j],2)) {
       if(argc>=j+1 && stype < 0) {
         stype = -2;
         if(!strncasecmp("exp",argv[j+1],3))stype = USMOKE_EXPLOSION;
         if(!strncasecmp("fir",argv[j+1],3))stype = USMOKE_FIRE;
         if(!strncasecmp("dus",argv[j+1],3))stype = USMOKE_DUST;
         if(!strncasecmp("mis",argv[j+1],3))stype = USMOKE_MISSLE;
         if(!strncasecmp("smo",argv[j+1],3))stype = USMOKE_SMOKE;
       }
     }
   /* TIME DIALATION */
     if(!strncasecmp("-warp",argv[j],5)) {
       if(argc>=j+1) {
          if(warp_factor = atof(argv[j+1]))
          if(warp_factor>5.0) {
             warp_factor=5.0;
             fprintf(stderr,"%s: time-warp speed maximum (5.0) is set\n",argv[0]);
          }
          if(warp_factor<1.0) {
             warp_factor=1.0;
             fprintf(stderr,"%s: time-warp speed minimum (1.0) is set\n",argv[0]);
          }
       }
     }
   /* COLORS (bgn,end) of SMOKE */
     if(!strncasecmp("-bgn",argv[j],4)) {
       if(argc>=j+3) {
          float r=atof(argv[j+1]),g=atof(argv[j+2]),b=atof(argv[j+3]);
          uSetVec3(red,r,g,b);
          if(r>1||g>1||b>1)
             (void)uNormalizeVec3(red);
          bset=1;
       }
     }
     if(!strncasecmp("-end",argv[j],4)) {
       if(argc>=j+3) {
          float r=atof(argv[j+1]),g=atof(argv[j+2]),b=atof(argv[j+3]);
          uSetVec3(blu,r,g,b);
          if(r>1||g>1||b>1)
             (void)uNormalizeVec3(blu);
          eset=1;
       }
     }
     if(!strncasecmp("-whi",argv[j],4)) {
        bkgd = 0xffffffff;
     }
     if(!strncasecmp("-eye",argv[j],4)) {
       if(argc>=j+3) {
          float r=atof(argv[j+1]),g=atof(argv[j+2]),b=atof(argv[j+3]);
          uSetVec3(eye,r,g,b);
          if(r>1||g>1||b>1)
             (void)uNormalizeVec3(eye);
       }
     }
     if(!strncasecmp("-c",argv[j],2)) {
       if(argc>=j+1) {
          k = atoi(argv[j+1]);
       }
     }
     if(!strncasecmp("-dir",argv[j],4)) {
       if(argc>=j+3) {
	 float r=atof(argv[j+1]),g=atof(argv[j+2]),b=atof(argv[j+3]);
          uSetVec3(dirvec,r,g,b);
          if(r>1||g>1||b>1)
             (void)uNormalizeVec3(dirvec);
       }
     }
     if(!strncasecmp("-den",argv[j],4)) {
       if(argc>=j+1) {
	 float r=atof(argv[j+1]);
         density = r;
         set_dense = 1;
       }
     }
     if(!strncasecmp("-exp",argv[j],4)) {
       if(argc>=j+1) {
	 float r=atof(argv[j+1]);
         expand = r;
         set_dense = 1;
       }
     }
     if(!strncasecmp("-dis",argv[j],4)) {
       if(argc>=j+1) {
	 float r=atof(argv[j+1]);
         dissipate = r;
         set_dense = 1;
       }
     }
     if(!strncasecmp("-spe",argv[j],4)) {
       if(argc>=j+1) {
	 float r=atof(argv[j+1]);
         sspeed = r;
       }
     }
   } /* end for-loop */

   if(stype==-2) fprintf(stderr,
     "%s: bad value to '-t' argument: must be one of\n\t\tEXPLOSION FIRE DUST MISSLE SMOKE\n", argv[0]);

    initialize();
    def_simple_light_calc();         /* use light definitions set up */
    use_simple_light_calc();

    forg[0] = 0.0; forg[1] = 0.0; forg[2] = 0.05;
    uInitSmokes();


for(i=0;i<k;i++) {
    tmp_smoke = uNewSmoke();
    tmp_fire  = uNewSmoke();
    
    uSmokeMode(tmp_smoke,USMOKE_START);
    uSmokeMode(tmp_fire ,USMOKE_START);
    uSmokeType(tmp_smoke,stype);
    uSmokeType(tmp_fire, USMOKE_FIRE);
    uSmokeOrigin(tmp_fire, forg, .9);

    if(bset||eset) {
       uSmokeColor(tmp_smoke, red, blu);
    }
    uSmokeDir(tmp_smoke,dirvec);
    if(set_dense){
	 uSmokeDensity(tmp_smoke, density, dissipate, expand);
    }
    if(sspeed != 1.73)
         tmp_smoke->speed = sspeed;
}
    while (TRUE)
    {
        while (qtest() || !attached)
        {
            dev = qread (&value);
            switch(dev)
            {
                case XKEY: 
		    if(value)
		       if(!X) X = 1; 
		       else   X = 0;
		    break;
                case YKEY: 
		    if(value)
		       if(!Y) Y = 1; 
		       else   Y = 0;
		    break;
                case ZKEY: 
		    if(value)
		       if(!Z) Z = 1; 
		       else   Z = 0;
		    break;
                case ESCKEY:
                    exit(0);
                    break;
                case REDRAW:
                    reshapeviewport();
                    drawscene();
                    break;
                case INPUTCHANGE:
                    attached = value;        
                    break;
            }
        }  
        drawscene();
    }   
}

/*        The initialize subroutine positions the window and specifies
 *        its future constraints.  The program is in double buffer
 *        and RGB mode.  The torus is initialized with parameters
 */ 

initialize()
{
    int gid1;
    float radc, radt;

    foreground();
    keepaspect(1,1);
    gid1 = winopen ("smokin'");

    doublebuffer();
    RGBmode();
    gconfig();
    zbuffer(TRUE);
    subpixel(TRUE);

    qdevice (ESCKEY);
    qdevice (XKEY);
    qdevice (YKEY);
    qdevice (ZKEY);
    qdevice (RIGHTMOUSE);

    mmode(MVIEWING);
    perspective(450, 1.0, 0.1, 100.0);
    loadmatrix(ident4);
    translate(0.0, 0.0, -10.0);

#if NOTEXTURE
#else
    inittex(); /* define all the texture stuff! */
#endif

}

float xa=0,ya=0,za=0;
drawscene()
{
    czclear(bkgd,getgdesc(GD_ZMAX));
    pushmatrix();
        if(X) {rot(xa+=5, 'x');}
        if(Y) {rot(ya+=5, 'y');}
        if(Z) {rot(za+=5, 'z');}


#if for_test_only
        drawsaddle(); 
#endif

    uDrawSmokes(eye);

    popmatrix();
    swapbuffers();
}

#if for_test_only
drawsaddle()
{
    float norm[3];

norm[0]=0; norm[1]=0; norm[2]=1;
    bgnpolygon();
      n3f(norm);
      v3f(vees);
      t2f(tees);
      v3f(vees+3);
      t2f(tees+2);
      v3f(vees+6);
      t2f(tees+4);
      v3f(vees+9);
      t2f(tees+6);
    endpolygon();

}
#endif /* for_test_only */
