/* uSmoke.h */
#ifndef _USMOKE_H_
#define _USMOKE_H_ 1
/*
 * Copyright (c) 1993, 1994, Silicon Graphics, Inc.
 *
 * Permission to use, copy, modify, distribute, and sell this software
 * and its documentation for any purpose is hereby granted without
 * fee, provided that (i) the above copyright notices and this
 * permission notice appear in all copies of the software and related
 * documentation, and (ii) the name of Silicon Graphics may not be
 * used in any advertising or publicity relating to the software
 * without the specific, prior written permission of Silicon Graphics.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.
 * 
 * IN NO EVENT SHALL SILICON GRAPHICS BE LIABLE FOR ANY SPECIAL,
 * INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY
 * THEORY OF LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE
 * OR PERFORMANCE OF THIS SOFTWARE.
 * 
 */

#ifdef DEBUG
#define dprintf(s,f,v) fprintf(s,f,v)
#else
#define dprintf(s,f,v)
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <gl/gl.h>
typedef float uVec2[2];
typedef float uVec3[3];
typedef float uVec4[4];
typedef float uMatrix[4][4];

#include "uTex.h"
#include "ufctimes.h"


#define MAX_SMOKE	16
#define MAX_PUFFS	32
#define MAX_RAND	256	/* Must be power of 2 */


#define USMOKE_BILLOW         0
#define USMOKE_EXPLOSION      1
#define USMOKE_FIRE           2       /* yes */
#define USMOKE_SMOKE          3       /* yes */
#define USMOKE_EXHAUST        4
#define USMOKE_DUST           5       /* yes */
#define USMOKE_MISSLE         6       /* yes */
#define USMOKE_SMOKE_FIRE     7

#define USMOKE_NUM_TYPES      8

#define USMOKE_STOP           0
#define USMOKE_START          1


#define RAND_VEC3	RandVec3[random() & (MAX_RAND - 1)]

#define MAXSMOKES	256

typedef struct _uPuff
{
    float	t;
    float	transp;
    double	startTime;

    float	radius;
    uVec3	origin;

    uMatrix	rotMat;

    uVec3       coords[4];
    uVec3       tcoords[4];

    float	speed;
    uVec3	direction;	/* Direction of travel with random factor */

} uPuff;

typedef struct _uSmoke
{
    uVec3	bgnColor, deltaColor;

    float	dissipation;	/* Dissipation rate in seconds */
    float	density;
    float	puffInterval;
    float	duration;

    double	startTime;
    double	prevTime;
    double	lastPuffTime;	

    long	type;
    long	mode;
    float	frequency;

    uVec3	direction;	/* Direction of travel */
    float	speed;
    float	turbulence;

    uVec3	origin;
    float	radius;
    float	expansion;
    float	offset;		
    
    uTexture	*tex;

    uPuff	*puffs[MAX_PUFFS];
    long	startPuff;
    long	numPuffs;

}uSmoke;

/*prototypes...*/
void uInitSmokes(void);
uSmoke* uNewSmoke(void);
void uSmokeOrigin(uSmoke *smoke, uVec3 origin, float radius);
void uSmokeMode(uSmoke *smoke, long mode);
void uSmokeDuration(uSmoke *smoke, float duration);
void uSmokeDir(uSmoke *smoke, uVec3 dir);
void uSmokeVelocity(uSmoke *smoke, float turbulence, float speed);
void uGetSmokeVelocity(uSmoke *smoke, float *turbulence, float *speed);
void uSmokeColor(uSmoke *smoke, uVec3 bgn, uVec3 end);
void uSmokeTex(uSmoke *smoke, uTexture *tex);
void uSmokeDensity(uSmoke *smoke, float dens, float diss, float expansion);
void uGetSmokeDensity(uSmoke *smoke, float *dens, float *diss, float *expansion);
void uSmokeType(uSmoke *smoke, long type);
void uDrawSmokes(uVec3 eye);
#include "smoke.h"   /* extra prototypes */

#endif /* _USMOKE_H_ */
